==> audiochain-1.1.5-src.tgz: org/audiochain/ui/ProjectMixdown.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashSet;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.swing.filechooser.FileFilter;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.AudioDataWriter;
import org.audiochain.io.FileAudioDataWriter;
import org.audiochain.io.FrameObserver;
import org.audiochain.io.MixingAudioDataReader;
import org.audiochain.io.Omitable;
import org.audiochain.io.RawAudioDataWriter;
import org.audiochain.io.RawFileAudioDataReader;
import org.audiochain.model.AudioMixer;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioTrack;


public class ProjectMixdown {
	public static enum AudioFileFormatType {
		Raw(null, RawFileAudioDataReader.FILE_NAME_ENDING), Aifc(AudioFileFormat.Type.AIFC, ".aifc"), Aiff(AudioFileFormat.Type.AIFF, ".aiff"), Au(AudioFileFormat.Type.AU, ".au"), Snd(
				AudioFileFormat.Type.SND, ".snd"), Wave(AudioFileFormat.Type.WAVE, ".wav");
		final AudioFileFormat.Type type;
		final String fileEnding;

		private AudioFileFormatType(AudioFileFormat.Type type, String fileEnding) {
			this.type = type;
			this.fileEnding = fileEnding;
		}

		public AudioFileFormat.Type getType() {
			return type;
		}

		public String getFileEnding() {
			return fileEnding;
		}

		public static AudioFileFormatType valueOf(File file) {
			String name = file.getName();
			for (AudioFileFormatType audioFileFormatType : AudioFileFormatType.values()) {
				if (name.endsWith(audioFileFormatType.getFileEnding())) {
					return audioFileFormatType;
				}
			}
			throw new IllegalArgumentException(AudioFileFormatType.class.getName() + " is not supported for file name " + name);
		}
	}

	private final AudioProject project;
	private Collection<ProjectMixdownListener> listeners;
	private boolean cancel;

	public ProjectMixdown(AudioProject project) {
		this.project = project;
	}

	public File createMixdown() throws IOException {
		return createMixdown(null, AudioFileFormatType.Raw, 0, -1);
	}

	public File createMixdown(File file, AudioFileFormatType audioFileFormatType, long startFrame, long endFrame) throws IOException {
		if (file == null) {
			file = createDefaultMixdownFile(null);
		}
		if (endFrame < 0) {
			endFrame = project.getMaxFrame();
		}
		if (startFrame < 0) {
			startFrame = 0;
		}
		if (startFrame > endFrame) {
			startFrame = endFrame;
		}
		long frameCount = endFrame - startFrame;
		if (frameCount <= 0) {
			return null;
		}
		AudioDataReader mix = createAudioDataReader();
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file));
		AudioFormat audioFormat = project.getAudioFormat();
		AudioDataWriter writer;

		switch (audioFileFormatType) {
		case Raw:
			writer = new RawAudioDataWriter(audioFormat, bos);
			break;
		case Aifc:
		case Snd:
		case Au:
			writer = new FileAudioDataWriter(audioFormat, audioFileFormatType.getType(), bos);
			break;
		case Aiff:
		case Wave:
			FileAudioDataWriter fileAudioDataWriter = new FileAudioDataWriter(audioFormat, audioFileFormatType.getType(), bos);
			fileAudioDataWriter.setPredictedLengthInFrames(endFrame-startFrame);
			writer = fileAudioDataWriter;
			break;
		default:
			return null;
		}

		mix.seek(startFrame);
		fireMixdownStarted(file, audioFileFormatType, startFrame, endFrame);

		final int[] sampleBuffer = new int[AudioMixer.getInstance().getPlaybackBufferSizeInSamples()];

		long readSamples = startFrame * 2;
		final long endSample = endFrame * 2;
		cancel = false;
		while (!cancel) {
			int sampleLen = mix.read(sampleBuffer);
			if (sampleLen == -1) {
				break;
			}
			if(readSamples+sampleLen>endSample) {
				sampleLen = (int) (endSample - readSamples);
			}
			writer.write(sampleBuffer, 0, sampleLen);
			
			readSamples += sampleLen;
			fireMixdownUpdated(readSamples / 2);
			if (readSamples >= endSample) {
				break;
			}
		}
		mix.stop();
		mix.close();
		writer.close();
		bos.close();
		fireMixdownFinished(readSamples / 2);
		return file;
	}

	public File createDefaultMixdownFile(AudioFileFormatType type) {
		if(type==null) {
			type = AudioFileFormatType.Raw;
		}
		File file;
		String name = project.getName();
		if (name == null) {
			name = "";
		} else {
			name = name.replace(' ', '_');
		}
		DateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
		file = project.getRelativeFile(name + "Mixdown" + df.format(new Date()) + type.getFileEnding());
		return file;
	}

	public void cancel() {
		cancel = true;
	}

	public AudioDataReader createAudioDataReader() throws IOException {
		return createAudioDataReader(null);
	}

	private AudioDataReader createAudioDataReader(FrameObserver frameObserver) throws IOException {
		Collection<AudioTrack> playbackTracks = project.getPlaybackTracks();
		if (playbackTracks.isEmpty()) {
			throw new IllegalStateException("No tracks for playback available.");
		}
		return createAudioDataReader(playbackTracks, frameObserver);
	}

	private AudioDataReader createAudioDataReader(Collection<AudioTrack> tracks, FrameObserver frameObserver) throws IOException {
		if (tracks.isEmpty()) {
			throw new IllegalArgumentException("There are no tracks.");
		}
		Collection<AudioDataReader> readers = new ArrayList<AudioDataReader>();
		for (AudioTrack track : tracks) {
			AudioDataReader base = track.getSourceAudioDevice().createAudioDataReader(project.getFrameRate(), project.getSampleSizeInBits(), frameObserver);
			if (base != null) {
				readers.add(track.getPlaybackAudioDeviceChain().createAudioDataReaderChain(base, project.getFrameRate(), project.getSampleSizeInBits(), frameObserver));
			}
		}
		
		for (AudioDataReader reader : readers) {
			omitRecursive(reader);
		}

		AudioDataReader chain = new MixingAudioDataReader(readers);
		AudioDataReader ret = project.getMasterAudioDeviceChain().createAudioDataReaderChain(chain, project.getFrameRate(), project.getSampleSizeInBits(), frameObserver);
		omitRecursive(ret);
		return ret;
	}
	
	private void omitRecursive(AudioDataReader reader) {
		if(reader instanceof Omitable) {
			((Omitable)reader).omit(true);
		}
		if(reader instanceof AudioDataReaderChainLink) {
			omitRecursive(((AudioDataReaderChainLink)reader).getSource());
		}
	}

	private void fireMixdownStarted(File file, AudioFileFormatType type, long startFrame, long endFrame) {
		if (listeners == null) {
			return;
		}
		for (ProjectMixdownListener listener : listeners) {
			listener.mixdownStarted(file, type, startFrame, endFrame);
		}
	}

	private void fireMixdownUpdated(long currentFrame) {
		if (listeners == null) {
			return;
		}
		for (ProjectMixdownListener listener : listeners) {
			listener.mixdownUpdated(currentFrame);
		}
	}

	private void fireMixdownFinished(long endFrame) {
		if (listeners == null) {
			return;
		}
		for (ProjectMixdownListener listener : listeners) {
			listener.mixdownFinished(endFrame);
		}
	}

	public void addProjectMixdownListener(ProjectMixdownListener listener) {
		if (listeners == null) {
			listeners = new LinkedHashSet<ProjectMixdownListener>();
		}
		listeners.add(listener);
	}

	public void removeProjectMixdownListener(ProjectMixdownListener listener) {
		if (listeners == null) {
			return;
		}
		listeners.remove(listener);
	}

	public static FileFilter createFileFilter() {
		return new FileFilter() {

			@Override
			public String getDescription() {
				StringBuilder sb = new StringBuilder();
				AudioFileFormatType[] values = ProjectMixdown.AudioFileFormatType.values();
				boolean first = true;
				for (AudioFileFormatType audioFileFormatType : values) {
					if (!first) {
						sb.append(", ");
					}
					sb.append('*');
					sb.append(audioFileFormatType.getFileEnding());
					first = false;
				}
				return sb.toString();
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return false;
				}
				String name = f.getName();

				for (AudioFileFormatType audioFileFormatType : AudioFileFormatType.values()) {
					if (name.endsWith(audioFileFormatType.getFileEnding())) {
						return true;
					}
				}

				return false;
			}
		};
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/ProjectMixdown.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashSet;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.swing.filechooser.FileFilter;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.AudioDataWriter;
import org.audiochain.io.FileAudioDataWriter;
import org.audiochain.io.FrameObserver;
import org.audiochain.io.LimitingAudioDataReader;
import org.audiochain.io.MixingAudioDataReader;
import org.audiochain.io.Omitable;
import org.audiochain.io.RawAudioDataWriter;
import org.audiochain.io.RawFileAudioDataReader;
import org.audiochain.model.AudioMixer;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioTrack;


public class ProjectMixdown {
	public static enum AudioFileFormatType {
		Raw(null, RawFileAudioDataReader.FILE_NAME_ENDING), Aifc(AudioFileFormat.Type.AIFC, ".aifc"), Aiff(AudioFileFormat.Type.AIFF, ".aiff"), Au(AudioFileFormat.Type.AU, ".au"), Snd(
				AudioFileFormat.Type.SND, ".snd"), Wave(AudioFileFormat.Type.WAVE, ".wav");
		final AudioFileFormat.Type type;
		final String fileEnding;

		private AudioFileFormatType(AudioFileFormat.Type type, String fileEnding) {
			this.type = type;
			this.fileEnding = fileEnding;
		}

		public AudioFileFormat.Type getType() {
			return type;
		}

		public String getFileEnding() {
			return fileEnding;
		}

		public static AudioFileFormatType valueOf(File file) {
			String name = file.getName();
			for (AudioFileFormatType audioFileFormatType : AudioFileFormatType.values()) {
				if (name.endsWith(audioFileFormatType.getFileEnding())) {
					return audioFileFormatType;
				}
			}
			throw new IllegalArgumentException(AudioFileFormatType.class.getName() + " is not supported for file name " + name);
		}
	}

	private final AudioProject project;
	private Collection<ProjectMixdownListener> listeners;
	private boolean cancel;

	public ProjectMixdown(AudioProject project) {
		this.project = project;
	}

	public File createMixdown() throws IOException {
		return createMixdown(null, AudioFileFormatType.Raw, 0, -1);
	}

	public File createMixdown(File file, AudioFileFormatType audioFileFormatType, long startFrame, long endFrame) throws IOException {
		if (file == null) {
			file = createDefaultMixdownFile(null);
		}
		if (endFrame < 0) {
			endFrame = project.getMaxFrame();
		}
		if (startFrame < 0) {
			startFrame = 0;
		}
		if (startFrame > endFrame) {
			startFrame = endFrame;
		}
		long frameCount = endFrame - startFrame;
		if (frameCount <= 0) {
			return null;
		}
		AudioDataReader mix = createAudioDataReader();
		BufferedOutputStream bos;
		AudioFormat audioFormat = project.getAudioFormat();
		AudioDataWriter writer;

		switch (audioFileFormatType) {
		case Raw:
			bos = new BufferedOutputStream(new FileOutputStream(file));
			writer = new RawAudioDataWriter(audioFormat, bos);
			break;
		case Aifc:
		case Snd:
		case Au:
			bos = new BufferedOutputStream(new FileOutputStream(file));
			writer = new FileAudioDataWriter(audioFormat, audioFileFormatType.getType(), bos);
			break;
		case Aiff:
		case Wave:
			bos = new BufferedOutputStream(new FileOutputStream(file));
			FileAudioDataWriter fileAudioDataWriter = new FileAudioDataWriter(audioFormat, audioFileFormatType.getType(), bos);
			fileAudioDataWriter.setPredictedLengthInFrames(endFrame-startFrame);
			writer = fileAudioDataWriter;
			break;
		default:
			return null;
		}

		mix.seek(startFrame);
		fireMixdownStarted(file, audioFileFormatType, startFrame, endFrame);

		final int[] sampleBuffer = new int[AudioMixer.getInstance().getPlaybackBufferSizeInSamples()];

		long readSamples = startFrame * 2;
		final long endSample = endFrame * 2;
		cancel = false;
		while (!cancel) {
			int sampleLen = mix.read(sampleBuffer);
			if (sampleLen == -1) {
				break;
			}
			if(readSamples+sampleLen>endSample) {
				sampleLen = (int) (endSample - readSamples);
			}
			writer.write(sampleBuffer, 0, sampleLen);
			
			readSamples += sampleLen;
			fireMixdownUpdated(readSamples / 2);
			if (readSamples >= endSample) {
				break;
			}
		}
		mix.stop();
		mix.close();
		writer.close();
		bos.close();
		fireMixdownFinished(readSamples / 2);
		return file;
	}

	public File createDefaultMixdownFile(AudioFileFormatType type) {
		if(type==null) {
			type = AudioFileFormatType.Raw;
		}
		File file;
		String name = project.getName();
		if (name == null) {
			name = "";
		} else {
			name = name.replace(' ', '_');
		}
		DateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
		file = project.getRelativeFile(name + "Mixdown" + df.format(new Date()) + type.getFileEnding());
		return file;
	}

	public void cancel() {
		cancel = true;
	}

	public AudioDataReader createAudioDataReader() throws IOException {
		return createAudioDataReader(null);
	}

	private AudioDataReader createAudioDataReader(FrameObserver frameObserver) throws IOException {
		Collection<AudioTrack> playbackTracks = project.getPlaybackTracks();
		if (playbackTracks.isEmpty()) {
			throw new IllegalStateException("No tracks for playback available.");
		}
		return createAudioDataReader(playbackTracks, frameObserver);
	}

	private AudioDataReader createAudioDataReader(Collection<AudioTrack> tracks, FrameObserver frameObserver) throws IOException {
		if (tracks.isEmpty()) {
			throw new IllegalArgumentException("There are no tracks.");
		}
		Collection<AudioDataReader> readers = new ArrayList<AudioDataReader>();
		for (AudioTrack track : tracks) {
			AudioDataReader base = track.getSourceAudioDevice().createAudioDataReader(project.getFrameRate(), project.getSampleSizeInBits(), frameObserver);
			if (base != null) {
				readers.add(track.getPlaybackAudioDeviceChain().createAudioDataReaderChain(base, project.getFrameRate(), project.getSampleSizeInBits(), frameObserver));
			}
		}
		
		for (AudioDataReader reader : readers) {
			omitRecursive(reader);
		}

		AudioDataReader chain = new MixingAudioDataReader(readers);
		AudioDataReader ret = project.getMasterAudioDeviceChain().createAudioDataReaderChain(chain, project.getFrameRate(), project.getSampleSizeInBits(), frameObserver);
		omitRecursive(ret);
		LimitingAudioDataReader limiter = new LimitingAudioDataReader(ret, project.getSampleSizeInBits());
		return limiter;
	}
	
	private void omitRecursive(AudioDataReader reader) {
		if(reader instanceof Omitable) {
			((Omitable)reader).omit(true);
		}
		if(reader instanceof AudioDataReaderChainLink) {
			omitRecursive(((AudioDataReaderChainLink)reader).getSource());
		}
	}

	private void fireMixdownStarted(File file, AudioFileFormatType type, long startFrame, long endFrame) {
		if (listeners == null) {
			return;
		}
		for (ProjectMixdownListener listener : listeners) {
			listener.mixdownStarted(file, type, startFrame, endFrame);
		}
	}

	private void fireMixdownUpdated(long currentFrame) {
		if (listeners == null) {
			return;
		}
		for (ProjectMixdownListener listener : listeners) {
			listener.mixdownUpdated(currentFrame);
		}
	}

	private void fireMixdownFinished(long endFrame) {
		if (listeners == null) {
			return;
		}
		for (ProjectMixdownListener listener : listeners) {
			listener.mixdownFinished(endFrame);
		}
	}

	public void addProjectMixdownListener(ProjectMixdownListener listener) {
		if (listeners == null) {
			listeners = new LinkedHashSet<ProjectMixdownListener>();
		}
		listeners.add(listener);
	}

	public void removeProjectMixdownListener(ProjectMixdownListener listener) {
		if (listeners == null) {
			return;
		}
		listeners.remove(listener);
	}

	public static FileFilter createFileFilter() {
		return new FileFilter() {

			@Override
			public String getDescription() {
				StringBuilder sb = new StringBuilder();
				AudioFileFormatType[] values = ProjectMixdown.AudioFileFormatType.values();
				boolean first = true;
				for (AudioFileFormatType audioFileFormatType : values) {
					if (!first) {
						sb.append(", ");
					}
					sb.append('*');
					sb.append(audioFileFormatType.getFileEnding());
					first = false;
				}
				return sb.toString();
			}

			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) {
					return false;
				}
				String name = f.getName();

				for (AudioFileFormatType audioFileFormatType : AudioFileFormatType.values()) {
					if (name.endsWith(audioFileFormatType.getFileEnding())) {
						return true;
					}
				}

				return false;
			}
		};
	}
}
